#!/usr/bin/env python

# fade_frames.py Release 1
# Created by Tin Tran http://gimplearn.net
#
# License: GPLv3
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY# without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# To view a copy of the GNU General Public License
# visit: http://www.gnu.org/licenses/gpl.html
#
# ------------
#| Change Log |
# ------------
# Rel 1: Initial release - based on plugins by Tin Tran:
# 		 Fade Frame Plug-in                                 https://gimplearn.net/viewtopic.php?f=3&t=2369&sid=cd6929e9f014606b69ce34aac81dc6f9
# 		 Slider Animation Script for GIMP (Swipe Animation) https://gimplearn.net/viewtopic.php?f=3&t=1037&hilit=slider
# 		 Rename Layers Script For Animated .gifs animation  https://gimplearn.net/viewtopic.php?f=3&t=72&p=4243&hilit=rename+layers#p4243
# Rel 2: Add: Slider Bar(from Top to Bottom), Ripple+Fade, Slider - Type of Fill, GroupLayers(frames), INTERACTIVE MathMap 
# Rel 3: Add: Page Curl effect with which corner option (fade_frames.py version 10. Allow user to choose which corner to peel page curl from.)
# Rel 4: Add: MathMap Vertical Gate, Rectangle Distress, Triangle Distress
# Rel 5: Add option: Reverse frames order, Add efect: Scrolling

from gimpfu import *
import random
import math
import os,re
import string
from array import array

global applylayers
applylayers = []
def flip_horizontally_list_of_coords(layerwidth,layerheight,coords):
	ret = []
	for i in range(0,len(coords)):
		if i % 2 == 0: #x coord
			x = coords[i]
			x = layerwidth - x #flip horizontally
			ret.append(x)
		else: #y coord
			y = coords[i]
			ret.append(y)
	ret = tuple(ret)
	return ret

def flip_horizontally_transformation(layerwidth,layerheight,x1,y1,x2,y2,x3,y3,x4,y4):
	x1,y1,x2,y2,x3,y3,x4,y4 = flip_horizontally_list_of_coords(layerwidth,layerheight,[x1,y1,x2,y2,x3,y3,x4,y4])
	x1,y1,x2,y2 = x2,y2,x1,y1 #swap them
	x3,y3,x4,y4 = x4,y4,x3,y3
	return x1,y1,x2,y2,x3,y3,x4,y4

def flip_vertically_list_of_coords(layerwidth,layerheight,coords):
	ret = []
	for i in range(0,len(coords)):
		if i % 2 == 0: #x coord
			x = coords[i]
			ret.append(x)
		else: #y coord
			y = coords[i]
			y = layerheight - y #flip vertically
			ret.append(y)
	ret = tuple(ret)
	return ret

def flip_vertically_transformation(layerwidth,layerheight,x1,y1,x2,y2,x3,y3,x4,y4):
	x1,y1,x2,y2,x3,y3,x4,y4 = flip_vertically_list_of_coords(layerwidth,layerheight,[x1,y1,x2,y2,x3,y3,x4,y4])
	x1,y1,x3,y3 = x3,y3,x1,y1 #swap them
	x2,y2,x4,y4 = x4,y4,x2,y2
	return x1,y1,x2,y2,x3,y3,x4,y4

def python_tt_fade_frames(image, layer, frames, outdir, filename, corner):
	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()	
def python_fu_slideshow_frames(image, layer, style, frames, slider_width, fill, prefix, delay, bl, tl, rev, disp, group, lg_name, play):

	pdb.gimp_image_undo_group_start(image)
	pdb.gimp_context_push()
	
	counter = 0
	if style==0 or style==5 or style==6 or style==7 or style==8:
		pdb.gimp_progress_init("exporting images...",None)
		fraction = 100.0/frames
		pdb.gimp_image_set_active_layer(image,layer)
		new_layer = pdb.gimp_layer_new_from_drawable(layer, image)
		pdb.gimp_image_insert_layer(image,new_layer,None,-1)
		pdb.gimp_item_set_visible(new_layer, 0)
		new_image = pdb.gimp_image_new(image.width, image.height, 0)
		display = pdb.gimp_display_new(new_image)
		start = pdb.gimp_layer_new_from_visible(image,new_image,"Fade_frames")
		pdb.gimp_image_insert_layer(new_image,start,None,0)
		for frame in range(0,int(frames)+1):
			opacity = frame * fraction
			pdb.gimp_progress_update(float(frame)/frames)
			pdb.gimp_layer_set_opacity(layer,opacity)
			
			if style==5:
				pdb.plug_in_cubism(new_image, layer, 10, 5, 0, run_mode=RUN_NONINTERACTIVE)
			if style==6:	
				pdb.plug_in_pixelize2(new_image, layer, 10, 10, run_mode=RUN_NONINTERACTIVE)
			if style==7:				
				pdb.plug_in_video(new_image, layer, 3, 1, 0, run_mode=RUN_NONINTERACTIVE)
			if style==8:				
				pdb.plug_in_ripple(new_image, layer, 20, 5, 0, 1, 1, 1, 0, run_mode=RUN_NONINTERACTIVE)
				#x = opacity- 100/(opacity+1)
				#pdb.plug_in_light_to_height(new_image, layer, layer, x, 10, 0, 0, 0, 0,run_mode=RUN_NONINTERACTIVE)
			visible = pdb.gimp_layer_new_from_visible(image,new_image,"Fade_frames")
			pdb.gimp_image_insert_layer(new_image,visible,None,0)
		pdb.gimp_item_set_visible(new_layer, 1)
		pdb.gimp_image_remove_layer(image, layer)
		pdb.gimp_image_raise_item_to_top(new_image, start)
		for layer in new_image.layers:	
			pdb.gimp_image_raise_item_to_top(new_image, layer)
	if style==1 or style==2 or style==3 or style==4:
		frames=int(frames)+1
		new_image = pdb.gimp_image_duplicate(image)
		pdb.gimp_display_new(new_image)
		layers_count = len(new_image.layers)
		animate_layers = list(new_image.layers)
		for layer in animate_layers:
			if not layer.has_alpha:
				pdb.gimp_layer_add_alpha(layer)
		
		bottom_layer = pdb.gimp_layer_new_from_drawable(new_image.layers[0],new_image)
		pdb.gimp_image_insert_layer(new_image,bottom_layer,None,layers_count)
		if style==1: # from Left to Right
			if slider_width > 0:
				slider_layer = pdb.gimp_layer_new(new_image,slider_width,image.height,RGBA_IMAGE,"slider",100,NORMAL_MODE)
		if style==2:
			if slider_width > 0:
				slider_layer = pdb.gimp_layer_new(new_image,image.width,slider_width,RGBA_IMAGE,"slider",100,NORMAL_MODE)				
		elif style  == 3 or style == 4: #circle
			slider_layer = pdb.gimp_layer_new(new_image,image.width,image.height,RGBA_IMAGE,"slider",100,NORMAL_MODE)
		if slider_width > 0:	
			pdb.gimp_image_insert_layer(new_image,slider_layer,None,0)

			pdb.gimp_edit_fill(slider_layer,fill) #type of fill { FILL-FOREGROUND (0), FILL-BACKGROUND (1), FILL-WHITE (2), FILL-TRANSPARENT (3), FILL-PATTERN (4) 
		
		if style==1 or style==2:
			for x in range(0,int(frames+1)):
			
				if style==1: # from Left to Right
					start = -slider_width
					inc = float(image.width+slider_width)/frames
					current_frame = animate_layers[0]
					pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,0,0,min(image.width,max(0,x*inc)),image.height)
					non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
				if style==2:  # from Top to Bottom
					start = -slider_width
					inc = float(image.height+slider_width)/frames	
					#for i in range(0,len(animate_layers)):
					current_frame = animate_layers[0]				
					pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,0,0,image.width,min(image.height,max(0,x*inc)))				
					non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
				if non_empty:
					pdb.gimp_edit_clear(current_frame)
				if slider_width > 0:
					if style==1:
						pdb.gimp_layer_set_offsets(slider_layer,start + x*inc,0)
					if style==2:
						pdb.gimp_layer_set_offsets(slider_layer,0,start + x*inc,)
				frame_layer = pdb.gimp_layer_new_from_visible(new_image,new_image,"frame")
				pdb.gimp_image_insert_layer(new_image,frame_layer,None,0)
				frame_layer.visible = False
			pdb.gimp_image_remove_layer(new_image,bottom_layer)
			if slider_width > 0:
				pdb.gimp_image_remove_layer(new_image,slider_layer)
			for layer in animate_layers:
				pdb.gimp_image_remove_layer(new_image,layer)
			
			for layer in new_image.layers:
				layer.visible = True			
		if style==3 or style==4:
			radius = ((image.width/2)**2 + (image.height/2)**2)**0.5
			start = -slider_width
			inc = float(radius+slider_width)/frames
			#for i in range(0,len(animate_layers)):
			current_frame = animate_layers[0]
			for x in range(0,int(frames+1)):
				pdb.gimp_selection_all(new_image)
				if slider_width > 0:
					pdb.gimp_edit_clear(slider_layer)
				if style==3:	
					pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,image.width/2-max(0,start+inc*x+slider_width),image.height/2-max(0,start+inc*x+slider_width),max(0,start+inc*x+slider_width)*2,max(0,start+inc*x+slider_width)*2)
					pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_SUBTRACT,image.width/2-max(0,start+inc*x),image.height/2-max(0,start+inc*x),max(0,start+inc*x)*2,max(0,start+inc*x)*2)					
					non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)				
				if style==4:	
					pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_REPLACE,image.width/2-max(0,start+inc*x+slider_width),image.height/2-max(0,start+inc*x+slider_width),max(0,start+inc*x+slider_width)*2,max(0,start+inc*x+slider_width)*2)
					pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_SUBTRACT,image.width/2-max(0,start+inc*x),image.height/2-max(0,start+inc*x),max(0,start+inc*x)*2,max(0,start+inc*x)*2)
					non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
				if non_empty:
					pdb.gimp_edit_fill(slider_layer,fill)
				if style==3:						
					pdb.gimp_image_select_rectangle(new_image,CHANNEL_OP_REPLACE,image.width/2-max(0,start+inc*x),image.height/2-max(0,start+inc*x),max(0,start+inc*x)*2,max(0,start+inc*x)*2)					
					non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
				if style==4:					
					pdb.gimp_image_select_ellipse(new_image,CHANNEL_OP_REPLACE,image.width/2-max(0,start+inc*x),image.height/2-max(0,start+inc*x),max(0,start+inc*x)*2,max(0,start+inc*x)*2)
					non_empty,x1,y2,x2,y2 = pdb.gimp_selection_bounds(new_image)
				if non_empty:
					pdb.gimp_edit_clear(current_frame)

				frame_layer = pdb.gimp_layer_new_from_visible(new_image,new_image,"frame")
				pdb.gimp_image_insert_layer(new_image,frame_layer,None,0)
				frame_layer.visible = False

			pdb.gimp_image_remove_layer(new_image,bottom_layer)
			if slider_width > 0:
				pdb.gimp_image_remove_layer(new_image,slider_layer)
			for layer in animate_layers:
				pdb.gimp_image_remove_layer(new_image,layer)
			
			for layer in new_image.layers:
				layer.visible = True	
	if style==9 or style==10 or style==11 or style==12 or style==13 or style==14 or style==15:
		new_image = pdb.gimp_image_new(image.width, image.height, 0)
		display = pdb.gimp_display_new(new_image)
		for layer in image.layers:
			pdb.gimp_image_set_active_layer(image,layer)		
			item_copy = pdb.gimp_layer_new_from_drawable(layer, new_image)
			pdb.gimp_image_insert_layer(new_image, item_copy, None, 0)
			pdb.gimp_drawable_update(item_copy, 0, 0, item_copy.width, item_copy.height)
			pdb.gimp_image_lower_item_to_bottom(new_image, item_copy)
		top_layer = new_image.layers[0]
		top_layer.name = "START"
		bottom_layer = new_image.layers[-1]
		bottom_layer.name = "END"

		if style==9:	
			pdb.mathmap_animation_angular_transition_bands(new_image,layer, 4, 10, 0, 10, run_mode=RUN_INTERACTIVE)
			for layer in new_image.layers:
				pdb.gimp_image_raise_item_to_top(new_image, layer)
			pdb.gimp_image_remove_layer(new_image, bottom_layer)
			pdb.gimp_image_lower_item_to_bottom(new_image, top_layer)
		if style==10:
			pdb.mathmap_animation_angular_transition_mathmap_by_nevit(new_image,layer, 4, 10, 0, 10, run_mode=RUN_INTERACTIVE)
			#pdb.gimp_image_remove_layer(new_image, bottom_layer)
			pdb.gimp_image_raise_item_to_top(new_image, bottom_layer)
		if style==11:			
			pdb.mathmap_animation_radial_transition_bands(new_image,layer, 4, 10, 0, 10, run_mode=RUN_INTERACTIVE)
			pdb.gimp_image_raise_item_to_top(new_image, top_layer)
			for layer in new_image.layers:
				pdb.gimp_image_raise_item_to_top(new_image, layer)
		if style==12:	
			pdb.mathmap_animation_ihorizontal_curtain_band_transition(new_image,layer, 4, 10, 0, 10, run_mode=RUN_INTERACTIVE)
			pdb.gimp_image_raise_item_to_top(new_image, bottom_layer)				
		if style==13:					
			pdb.mathmap_animation_vertical_curtain_transition_bands(new_image,layer, 4, 10, 0, 10, run_mode=RUN_INTERACTIVE)
			pdb.gimp_image_raise_item_to_top(new_image, bottom_layer)
		if style==14:
			pdb.mathmap_animation_zoom_and_rotate_by_nevit(new_image,layer, 4, 10, 0, 10, run_mode=RUN_INTERACTIVE)
			bottom_layer = new_image.layers[-1]
			pdb.gimp_image_raise_item_to_top(new_image, bottom_layer)
			pdb.gimp_image_raise_item_to_top(new_image, top_layer)			
			rest_of_layers = list(new_image.layers[1:])
			top_layer_new = new_image.layers[0]
			for i in range(0,len(rest_of_layers)):
				pdb.gimp_item_set_visible(layer, 1)	
				cur_layer = rest_of_layers[i]
				position = pdb.gimp_image_get_item_position(new_image, cur_layer) #grabs position
				copy = pdb.gimp_layer_new_from_drawable(top_layer_new, new_image) #copy of top layer
				pdb.gimp_image_insert_layer(new_image, copy, None, position)  #insert into position so that it's on top of existing
				pdb.gimp_image_lower_item(new_image, copy)	
				result_layer = pdb.gimp_image_merge_down(new_image, cur_layer, EXPAND_AS_NECESSARY) #merge down
			pdb.gimp_image_remove_layer(new_image,top_layer_new)
		if style==15:
			pdb.mathmap_animation_vertical_gate_transition(new_image,layer, 4, 10, 0, 10, run_mode=RUN_INTERACTIVE)
			pdb.gimp_image_remove_layer(new_image, bottom_layer)
			for layer in new_image.layers:
				pdb.gimp_image_raise_item_to_top(new_image, layer)
			pdb.gimp_image_lower_item_to_bottom(new_image, top_layer)			
			
	if style==16 or style==17 or style==18 or style==19:
	
		pdb.gimp_progress_init("Page Curling...",None)
		steps = (int(frames)+1)
		width_step = (layer.width)/steps
		fra = -1
		zstart = 150.0
		zend = 500.0
		zstep = (zend - zstart)/steps
		angle = -50
		angle_r = math.radians(angle)
		x_factor = math.cos(angle_r)
		y_factor = math.sin(angle_r)
		radius = abs(float(layer.height)/y_factor)
		A = radius * x_factor
		O = radius * y_factor
		
		pdb.gimp_image_set_active_layer(image,layer)
		new_layer = pdb.gimp_layer_new_from_drawable(layer, image)
		pdb.gimp_image_insert_layer(image,new_layer,None,-1)
		pdb.gimp_item_set_visible(new_layer, 0)
		new_image = pdb.gimp_image_new(image.width, image.height, 0)
		display = pdb.gimp_display_new(new_image)
		start = pdb.gimp_layer_new_from_visible(image,new_image,"Fade_frames")
		pdb.gimp_image_insert_layer(new_image,start,None,0)		
		x_step = float(layer.width + A)/steps
		for frame in range(0,steps+1):
			fra += 1
			pdb.gimp_progress_update(float(frame)/steps)

			pdb.gimp_item_set_visible(layer,TRUE)
			layer_copy = pdb.gimp_layer_new_from_drawable(layer,image)
			pdb.gimp_image_insert_layer(image,layer_copy,None,0)
			pdb.gimp_item_set_visible(layer_copy,TRUE)
			pdb.gimp_item_set_visible(layer,FALSE)
			
			x = layer.width - (frame*x_step)
			y = layer.height
			if x >= layer.width - A: #bottom right corner curl

				small_radius = float(layer.width-x)/x_factor
				x1 = layer.width
				a = small_radius * x_factor
				o = small_radius * y_factor
				y1 = layer.height + o
				x2 = layer.width
				y2 = layer.height
				t_radius = x2 - x
				reflect_angle = -100
				reflect_angle_r = math.radians(reflect_angle)
				x2_ = x + t_radius*math.cos(reflect_angle_r)
				y2_ = y + t_radius*math.sin(reflect_angle_r)
				x3_ = x1 - t_radius*math.cos(reflect_angle_r)
				y3_ = y1 - t_radius*math.sin(reflect_angle_r)
				cx1,cy1,cx2,cy2,cx3,cy3 = x,y,x1,y1,x2,y2
				if style==17 or style==18:
					cx1,cy1,cx2,cy2,cx3,cy3 = flip_horizontally_list_of_coords(layer.width,layer.height,[cx1,cy1,cx2,cy2,cx3,cy3])
				if style==18 or style==19:
					cx1,cy1,cx2,cy2,cx3,cy3 = flip_vertically_list_of_coords(layer.width,layer.height,[cx1,cy1,cx2,cy2,cx3,cy3])
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_REPLACE,6,[cx1,cy1,cx2,cy2,cx3,cy3])
				non_empty,_,_,_,_ = pdb.gimp_selection_bounds(image)

				transformed = layer_copy
				if non_empty:
				
					cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = x3_,y3_,x1,y1,x,y,x2_,y2_
					if style==17 or style==18:
						cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = flip_horizontally_transformation(layer.width,layer.height,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)
					if style==18 or style==19:
						cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = flip_vertically_transformation(layer.width,layer.height,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)
					transformed = pdb.gimp_item_transform_perspective(layer_copy,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)

			elif x >= 0: #center curl
				#pdb.gimp_message("run")
				small_radius = float(layer.width-x)/x_factor
				x1 = x + A
				y1 = 0
				x2 = layer.width
				y2 = 0
				x3 = layer.width
				y3 = layer.height
				x_,y_ = x,y
				t_radius = x1 - x
				reflect_angle = -100
				reflect_angle_r = math.radians(reflect_angle)
				x1_ = x1 - t_radius*math.cos(reflect_angle_r)
				y1_ = y1 - t_radius*math.sin(reflect_angle_r)
				t_radius = x2-x1
				reflect_angle = -100
				reflect_angle_r = math.radians(reflect_angle)
				x2_ = x1 + t_radius*math.cos(reflect_angle_r)
				y2_ = y1 + t_radius*math.sin(reflect_angle_r)
				t_radius = x3-x
				reflect_angle = -100
				reflect_angle_r = math.radians(reflect_angle)
				x3_ = x + t_radius*math.cos(reflect_angle_r)
				y3_ = y + t_radius*math.sin(reflect_angle_r)
				cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = x,y,x1,y1,x2,y2,x3,y3
				
				if style==17 or style==18:
					cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = flip_horizontally_list_of_coords(layer.width,layer.height,[cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4])
				if style==18 or style==19:
					cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = flip_vertically_list_of_coords(layer.width,layer.height,[cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4])
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_REPLACE,8,[cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4])
				non_empty,_,_,_,_ = pdb.gimp_selection_bounds(image)
				transformed = layer_copy
				if non_empty:
					cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = x1_,y1_,x2_,y2_,x,y,x3_,y3_
					if style==17 or style==18:
						cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = flip_horizontally_transformation(layer.width,layer.height,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)
					if style==18 or style==19:
						cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = flip_vertically_transformation(layer.width,layer.height,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)
					transformed = pdb.gimp_item_transform_perspective(layer_copy,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)
			else: #top left corner curl
				x0 = 0
				y0 = 0
				x1 = x + A
				y1 = 0
				small_radius = float(x1)/x_factor
				x2 = 0
				a = small_radius * x_factor
				o = small_radius * y_factor
				y2 = abs(o)
				t_radius = x1
				reflect_angle = -100
				reflect_angle_r = math.radians(reflect_angle)
				x1_ = x1 - t_radius*math.cos(reflect_angle_r)
				y1_ = y1 - t_radius*math.sin(reflect_angle_r)
				t_radius = layer.width-x1
				reflect_angle = -100
				reflect_angle_r = math.radians(reflect_angle)
				x2_ = x1 + t_radius*math.cos(reflect_angle_r)
				y2_ = y1 + t_radius*math.sin(reflect_angle_r)
				t_radius = layer.height-y2
				reflect_angle = -190
				reflect_angle_r = math.radians(reflect_angle)
				x3_ = x2 + t_radius*math.cos(reflect_angle_r)
				y3_ = y2 + t_radius*math.sin(reflect_angle_r)
				t_radius = layer.width
				reflect_angle = -100
				reflect_angle_r = math.radians(reflect_angle)
				x4_ = x3_ + t_radius*math.cos(reflect_angle_r)
				y4_ = y3_ + t_radius*math.sin(reflect_angle_r)
				cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4,cx5,cy5 = x1,y1,layer.width,0,layer.width,layer.height,0,layer.height,x2,y2
				if style==17 or style==18:
					cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4,cx5,cy5 = flip_horizontally_list_of_coords(layer.width,layer.height,[cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4,cx5,cy5])
				if style==18 or style==19:
					cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4,cx5,cy5 = flip_vertically_list_of_coords(layer.width,layer.height,[cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4,cx5,cy5])
				pdb.gimp_image_select_polygon(image,CHANNEL_OP_REPLACE,10,[cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4,cx5,cy5])
				non_empty,_,_,_,_ = pdb.gimp_selection_bounds(image)

				transformed = layer_copy
				if non_empty:
					cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = x1_,y1_,x2_,y2_,x3_,y3_,x4_,y4_
					if style==17 or style==18:
						cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = flip_horizontally_transformation(layer.width,layer.height,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)
					if style==18 or style==19:
						cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4 = flip_vertically_transformation(layer.width,layer.height,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)
					transformed = pdb.gimp_item_transform_perspective(layer_copy,cx1,cy1,cx2,cy2,cx3,cy3,cx4,cy4)
			if pdb.gimp_layer_is_floating_sel(transformed):
				pdb.gimp_floating_sel_anchor(transformed)

			visible = pdb.gimp_layer_new_from_visible(image,new_image,"visible")
			pdb.gimp_image_insert_layer(new_image,visible,None,0)

			pdb.gimp_image_remove_layer(image, layer_copy)
		pdb.gimp_image_remove_layer(new_image, start)
		pdb.gimp_image_remove_layer(image, new_layer)
		pdb.gimp_item_set_visible(layer,TRUE)

	if style==20:
		pdb.gimp_selection_none(image)
		new_image = pdb.gimp_image_new(image.width, image.height, 0)
		pdb.gimp_display_new(new_image)

		for layer in image.layers:
			pdb.gimp_image_set_active_layer(image,layer)		
			item_copy = pdb.gimp_layer_new_from_drawable(layer, new_image)
			pdb.gimp_image_insert_layer(new_image, item_copy, None, 0)
			pdb.gimp_drawable_update(item_copy, 0, 0, item_copy.width, item_copy.height)
			pdb.gimp_image_lower_item_to_bottom(new_image, item_copy)
		top_layer = new_image.layers[0]
		top_layer.name = "START"
		bottom_layer = new_image.layers[-1]
		bottom_layer.name = "END"
		w = layer.width
		h = layer.height
		m = min(w, h)
	
		for frame in range(0,int(frames)+1):
			s = m/(frame+1)
			pdb.gimp_selection_all(new_image)
			top_layer = new_image.layers[0]
			visible = pdb.gimp_layer_new_from_drawable(top_layer, new_image)
			pdb.gimp_image_insert_layer(new_image,visible,None,0)
			pdb.gimp_selection_shrink(new_image, s, run_mode=RUN_NONINTERACTIVE)
			pdb.script_fu_distress_selection(new_image,visible,20, 10, 5, 5, 0, 0, run_mode=RUN_NONINTERACTIVE)
			pdb.gimp_selection_invert(new_image)
			pdb.gimp_edit_clear(visible)
			pdb.gimp_item_set_visible(top_layer, 0)
			pdb.gimp_item_set_visible(visible, 1)
			vis = pdb.gimp_layer_new_from_visible(new_image,new_image,"Frames x")
			pdb.gimp_image_insert_layer(new_image,vis,None,0)			
			pdb.gimp_image_lower_item_to_bottom(new_image, vis)
			pdb.gimp_image_remove_layer(new_image, visible)
			pdb.gimp_item_set_visible(top_layer, 1)
			pdb.gimp_selection_none(new_image)
		pdb.gimp_image_remove_layer(new_image, bottom_layer)
		pdb.gimp_image_lower_item_to_bottom(new_image, top_layer)
		
	if style==21:
		pdb.gimp_selection_none(image)
		new_image = pdb.gimp_image_new(image.width, image.height, 0)
		pdb.gimp_display_new(new_image)	
		selection = pdb.gimp_selection_bounds(new_image)
		x1 = selection[1];
		y1 = selection[2];
		x2 = selection[3];
		y2 = selection[4];
		for layer in image.layers:
			pdb.gimp_image_set_active_layer(image,layer)		
			item_copy = pdb.gimp_layer_new_from_drawable(layer, new_image)
			pdb.gimp_image_insert_layer(new_image, item_copy, None, 0)
			pdb.gimp_drawable_update(item_copy, 0, 0, item_copy.width, item_copy.height)
			pdb.gimp_image_lower_item_to_bottom(new_image, item_copy)
		top_layer = new_image.layers[0]
		top_layer.name = "START"
		bottom_layer = new_image.layers[-1]
		bottom_layer.name = "END"

		k = 90*0.01745329252
		
		for frame in range(0,int(frames)):
			top_layer = new_image.layers[0]
			visible = pdb.gimp_layer_new_from_drawable(top_layer, new_image)
			pdb.gimp_image_insert_layer(new_image,visible,None,0)
			
			w = new_image.width
			h = new_image.height
			s = k/(frame+1)

			pdb.gimp_image_select_polygon(new_image,CHANNEL_OP_ADD,8,[1+w/2,0, w,0, w,h, w/2,h])
			pdb.script_fu_distress_selection(new_image,visible,20, 10, 5, 5, 0, 0, run_mode=RUN_NONINTERACTIVE)
			pdb.gimp_item_transform_rotate(visible, s, 0, w/2, h)
			pdb.gimp_selection_invert(new_image)
			pdb.gimp_edit_clear(visible)
			pdb.gimp_item_set_visible(top_layer, 0)
			vis = pdb.gimp_layer_new_from_visible(new_image,new_image,"Frames p")
			pdb.gimp_image_insert_layer(new_image,vis,None,0)	
			pdb.gimp_floating_sel_anchor(pdb.gimp_image_get_floating_sel(new_image))
			pdb.gimp_image_remove_layer(new_image, visible)
			pdb.gimp_selection_none(new_image)
			
			visible = pdb.gimp_layer_new_from_drawable(top_layer, new_image)
			pdb.gimp_image_insert_layer(new_image,visible,None,0)
			pdb.gimp_image_select_polygon(new_image,CHANNEL_OP_ADD,8,[-1+w/2,0,  0,0, 0,h, w/2,h])
			pdb.script_fu_distress_selection(new_image,visible,20, 10, 5, 5, 0, 0, run_mode=RUN_NONINTERACTIVE)
			pdb.gimp_item_transform_rotate(visible, -s, 0, w/2, h)
			pdb.gimp_selection_invert(new_image)
			pdb.gimp_edit_clear(visible)
			pdb.gimp_floating_sel_anchor(pdb.gimp_image_get_floating_sel(new_image))
			
			pdb.gimp_item_set_visible(visible, 1)
			x = pdb.gimp_image_merge_down(new_image,visible,EXPAND_AS_NECESSARY)
			pdb.gimp_item_set_visible(top_layer, 0)	
			pdb.gimp_image_lower_item_to_bottom(new_image, x)
			pdb.gimp_item_set_visible(top_layer, 1)
			pdb.gimp_selection_none(new_image)	
		pdb.gimp_image_lower_item_to_bottom(new_image, top_layer)
	if style==22 or style==23 or style==24 or style==25 or style==26 or style==27 or style==28 or style==29:
		pdb.gimp_selection_none(image)
		new_image = pdb.gimp_image_new(image.width, image.height, 0)
		pdb.gimp_display_new(new_image)
		
		for layer in image.layers:
			pdb.gimp_image_set_active_layer(image,layer)		
			item_copy = pdb.gimp_layer_new_from_drawable(layer, new_image)
			pdb.gimp_image_insert_layer(new_image, item_copy, None, 0)
			pdb.gimp_drawable_update(item_copy, 0, 0, item_copy.width, item_copy.height)
			pdb.gimp_image_lower_item_to_bottom(new_image, item_copy)
		top_layer = new_image.layers[0]
		top_layer.name = "START"
		bottom_layer = new_image.layers[-1]
		bottom_layer.name = "END"

		w = new_image.width
		h = new_image.height
		for frame in range(0,int(frames)+1):
			
			bottom_img = image.layers[-1]
			visible = pdb.gimp_layer_new_from_drawable(bottom_img, new_image)
			pdb.gimp_image_insert_layer(new_image,visible,None,0)

			x = w/(frames+1)
			y = h/(frames+1)
			
			if style==22: # L -> R
				pdb.gimp_layer_translate(visible,-w+x*frame,0)
			if style==23: # R -> L
				pdb.gimp_layer_translate(visible,w-x*frame,0)
			if style==24: # T -> B
				pdb.gimp_layer_translate(visible,0, -h+y*frame)			
			if style==25: # B -> T
				pdb.gimp_layer_translate(visible,0, h-y*frame)			
			if style==26: # TL -> BR
				pdb.gimp_layer_translate(visible,-w+x*frame, -h+y*frame)
			if style==27: # BR -> TL
				pdb.gimp_layer_translate(visible,w-x*frame,  h-y*frame)
			if style==28: # BL -> TR
				pdb.gimp_layer_translate(visible,-w+x*frame,  h-y*frame)
			if style==29: # TR -> BL
				pdb.gimp_layer_translate(visible,w-x*frame,  -h+y*frame)			
				
			bot = pdb.gimp_layer_new_from_visible(new_image,new_image,"Frame ")
			pdb.gimp_image_insert_layer(new_image,bot,None,0)
			pdb.gimp_image_lower_item_to_bottom(new_image, bot)
			pdb.gimp_image_remove_layer(new_image, visible)

		r = pdb.gimp_image_merge_down(new_image,bottom_layer,EXPAND_AS_NECESSARY)
		pdb.gimp_image_lower_item_to_bottom(new_image, r)
		if rev==0:
			for layer in new_image.layers:
				pdb.gimp_image_raise_item_to_top(new_image, layer)


	
	applylayers = []
	counter = 0

	for layer in new_image.layers[::-1]:
		counter += 1
		applylayers.append([layer,counter])

	for applylayer in applylayers:
		applylayer[0].name = "Name"
		
	for applylayer in applylayers:
		layername = prefix + ' ' + str(applylayer[1]) + ' (' + str(int(delay)) + 'ms)'
		if disp == 1:
			layername = layername + ' (replace)'
		applylayer[0].name = layername
		if disp == 2:
			layername = layername + ' (combine)'
		applylayer[0].name = layername
		
	all = str(int(delay))
	top = str(int(tl))
	bot = str(int(bl))
	
	if tl>0:    # Start bottom layer = End/Top frame
		top_layer = new_image.layers[0]
		top_layer.name = top_layer.name.replace(all, top)		
	if bl>0:	## Start top layer = Start/Bottom frame
		bot_layer = new_image.layers[-1]
		bot_layer.name = bot_layer.name.replace(all, bot)
		
	if group==1:
	
		group_new = pdb.gimp_layer_group_new(new_image)
		parent = pdb.gimp_item_get_parent(group_new)
		position = pdb.gimp_image_get_item_position(new_image, layer)
		pdb.gimp_image_insert_layer(new_image,group_new, None, 0) 
		group_new.name = str(lg_name)
		applylayers = []
		for applylayer in new_image.layers[1:]:
			pdb.gimp_image_reorder_item(new_image, applylayer, group_new, position)
			pdb.gimp_image_lower_item_to_bottom(new_image, applylayer)	

	if play==1:
		pdb.plug_in_animationplay(new_image, layer, run_mode=RUN_NONINTERACTIVE)

	pdb.gimp_context_pop()
	pdb.gimp_image_undo_group_end(image)
	pdb.gimp_displays_flush()

		
register(
	"python_fu_slideshow_frames",                           
	"Transition from two layers in frames for slideshow: \nfrom \"Start\" TOP layer(ACTIVE) to \"End\" BOTTOM layer ",
	"Transition from 1 frame to another v.5",
	"Tin Tran/MrQ",
	"GPL",
	"May 2019",
	"<Image>/Filters/Animation/Animators/Slideshow...",             #Menu path
	"RGB*, GRAY*", 
	[
	(PF_OPTION, "style", " Efect : ", 1, [" Fade "," Slider Bar\n (from Left to Right) "," Slider Bar\n (from Top to Bottom)"," Slider Rectangle\n (from Inside to Outer) "," Slider Circlen\n (from Inside to Outer) ",
	" Cubism+Fade "," Pixelize+Fade "," Video+Fade "," Ripple+Fade ",
	" MATHMAP\n Angular Transition 1 "," MATHMAP\n Angular Transition 2 "," MATHMAP\n Radial Transition "," MATHMAP\n Horizontal Curtain ",
	" MATHMAP\n Vertical Curtain "," MATHMAP\n Zoom and Rotate "," MATHMAP\n Vertical Gate "," PAGE CURL\n from Bottom Right ","PAGE CURL\n from Bottom Left ","PAGE CURL\n from Top Left ","PAGE CURL\n from Top Right ",
	" RECTANGLE DISTRESS "," TRIANGLE DISTRESS ",
	" SCROLLING\n L -> R "," SCROLLING\n R -> L "," SCROLLING\n T -> B "," SCROLLING\n B -> T "," SCROLLING\n TL -> BR "," SCROLLING\n BR -> TL "," SCROLLING\n BL -> TR "," SCROLLING\n TR -> BL "]),
	(PF_SPINNER, "frames", " Number of Frames: ", 10, (1, 1000, 1)),
	(PF_SPINNER, "slider_width", " Slider - Width: ", 10, (0, 1000, 1)),
	(PF_OPTION, "fill", " Slider- Type of Fill: ", 0, [" FILL-FOREGROUND "," FILL-BACKGROUND "," FILL-WHITE "," FILL-TRANSPARENT "," FILL-PATTERN "]),	
	(PF_STRING, "prefix", " Layer Name Prefix: ", "Frame"),
	(PF_SPINNER, "delay", " Delay All Frames (ms): ", 100, (1, 10000, 1)),
	(PF_SPINNER, "bl", " Delay START Frame (ms):\n (if = 0 Start = All) ", 0, (0, 10000, 1)),	
	(PF_SPINNER, "tl", " Delay END Frame (ms):\n (if = 0 End = All) ", 0, (0, 10000, 1)),
	(PF_TOGGLE,  "rev", " Reverse frames order: ", 0 ),		
	(PF_OPTION, "disp", " Disposal Type: ", 1, [" DON'T CARE\n (*.webp) "," REPLACE\n (*.gif) "," COMBINE\n (*.gif) "]),
	(PF_TOGGLE,  "group", " Group frames:\n (Don't use  for playback\n or export as animation) ", 0 ),
	(PF_STRING, "lg_name", " Name group", "GIMPCHAT"),
	(PF_TOGGLE,  "play", " Animation play: ", 0 ),	
	],
	[],
	python_fu_slideshow_frames)

main()
